 /********************************************************************/
 /*                                                                  */
 /* Program name: mqcpcnvt                                           */
 /*                                                                  */
 /* Programmer: Tim Zielke                                           */
 /*                                                                  */
 /* Description: This program takes a specified byte string, ccsid,  */
 /*              and queue and puts that message on the queue with   */
 /*              that code page encoding. It then takes a second     */
 /*              ccsid and gets the message with the convert option  */
 /*              and displays what the byte string now looks like    */
 /*              under the new code page.                            */
 /* Inputs:                                                          */
 /*   1) input data to convert (hex, string, or file)                */
 /*   2) ccsid for put                                               */
 /*   3) ccsid for get with convert option                           */
 /*   4) queue to put/get from                                       */
 /*   5) queue manager (optional)                                    */
 /*   6) format (optional) (Default is MQSTR. Enter MQNONE for MQFMT_NONE) */
 /*   7) encoding (optional)                                         */ 
 /*                                                                  */
 /* Example:                                                         */
 /*   mqcpcnvt FD 819 1208 Q1                                        */ 
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
    /* includes for MQI */
 #include <cmqc.h>

#define BUFFERMAX 104857600 

long ConvertHexToDecLen2(char HexStr[]);

 int main(int argc, char **argv)
 {
   /*  Declare file and character for sample input                   */
   FILE *fp;

   /*   Declare MQI structures needed                                */
   MQOD     od = {MQOD_DEFAULT};    /* Object Descriptor             */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
      /** note, sample uses defaults where it can **/

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   MQLONG   GetMsgLen;              /* message length                */
   MQLONG   ByteLen;
   MQLONG   BufferLen;
   MQLONG   HoldCodedCharSetId;
   MQBYTE24 CorrelId;
   
   char     buffer[BUFFERMAX+1];   /* message buffer                */
   char     holdBytes[2];
   char     QMName[50];             /* queue manager name            */
   char     Format[MQ_FORMAT_LENGTH]; /* Format            */
   char    *pGetBuffer;
   char    *pChar;
   char     hexFlag='N';
   int i, j, k;
   FILE *fileptr;
   long filelen;
   int errnum;

   printf("mqcpcnvt start\n");

   if (argc < 5)
   {
     printf("Required inputs are the following:\n");
     printf("  1) input data to convert (e.g. 0x786d for hex, fox for string, input.txt for file)\n");
     printf("  2) ccsid for put\n"); 
     printf("  3) ccsid for get with convert option\n"); 
     printf("  4) queue to put/get from\n");
     printf("Optional inputs are the following:\n");
     printf("  5) queue manager\n");
     printf("  6) format (Default is MQSTR. Enter MQNONE for MQFMT_NONE)\n");
     printf("  7) encoding\n");
     printf("Example:\n");
     printf("  mqcpcnvt FD 819 1208 Q1\n");
     printf("Ending Program!\n");
     exit(-1);
   }

   fileptr = fopen(argv[1], "rb");
   if (fileptr != NULL)
   {
     printf("The input data was detected as file: %s\n", argv[1]);
     fseek(fileptr, 0, SEEK_END);
     filelen = ftell(fileptr);
     rewind(fileptr);
     if (filelen > BUFFERMAX)
       filelen=BUFFERMAX; 
     fread(buffer, filelen, 1, fileptr);
     fclose(fileptr);
     pChar=buffer;
     BufferLen=filelen;
   }
   else
   {
     pChar = argv[1];
     ByteLen = strlen(pChar);

     if (ByteLen > 2 && *pChar == '0' && *(pChar+1) == 'x') 
     {
       printf("The input data was detected as hex string: %s\n", argv[1]);
       hexFlag='Y';
       pChar=pChar+2;
       ByteLen=ByteLen-2;

       j=0;
       k=0;
       for (i=0; i < ByteLen; i++)
       {
         if (k > BUFFERMAX)
         {
            printf("Number of resolved chars in input hex string can not exceed %d!\n", BUFFERMAX);
            printf("Ending Program!\n");
            exit(-1);
         }

         if (*(pChar+i) != ' ')
         {
            holdBytes[j]=*(pChar+i);
            j++;
         }
         if (j == 2)
         {
            buffer[k]=ConvertHexToDecLen2(holdBytes);
            k++;
            j=0;
         }
       }

       if (j != 0)
       {
          printf("The input hex string does not contain an even amount of hex characters!\n");
          printf("Ending Program!\n");
          exit(-1);
       }
       buffer[k]='\0';
       BufferLen=k;
     }
     else
     {
       printf("The input data was detected as char string: %s\n", argv[1]);

       BufferLen=ByteLen;

       if (BufferLen > BUFFERMAX)
       {
         printf("Number of chars in input string can not exceed %d!\n", BUFFERMAX);
         printf("Ending Program!\n");
         exit(-1);
       }

       for (i=0; i < BufferLen; i++)
         buffer[i]=*(pChar+i);
     }
     buffer[i]='\0';

   }

   /******************************************************************/
   /*                                                                */
   /*   Connect to queue manager                                     */
   /*                                                                */
   /******************************************************************/
   QMName[0] = 0;    /* default */
   if (argc > 5)
     strncpy(QMName, argv[5], (size_t)MQ_Q_MGR_NAME_LENGTH);
   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %d\n", CReason);
     printf("Ending Program!\n");
     exit(-1);
   }

   /******************************************************************/
   /*                                                                */
   /*   Use parameter as the name of the target queue                */
   /*                                                                */
   /******************************************************************/
   strncpy(od.ObjectName, argv[4], (size_t)MQ_Q_NAME_LENGTH);
   printf("target queue is %s\n", od.ObjectName);

   O_options = MQOO_OUTPUT            /* open queue for output     */
             | MQOO_FAIL_IF_QUIESCING /* but not if MQM stopping   */
             | MQOO_INPUT_EXCLUSIVE 
            ;                        /* = 0x2010 = 8208 decimal   */

   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* MQOPEN completion code       */
          &Reason);                  /* reason code                  */

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN ended with reason code %d\n", Reason);
     printf("Ending Program!\n");
     goto MOD_EXIT; 
   }

   if (argc > 6)
   {
     if (!strcmp(argv[6],"MQNONE"))
     {
        memcpy(md.Format,           /* character string format            */
             MQFMT_NONE, (size_t)MQ_FORMAT_LENGTH);
     }
     else
     {
        strncpy(Format, argv[6], (size_t)MQ_FORMAT_LENGTH);
        memcpy(md.Format,           /* character string format            */
             Format, (size_t)MQ_FORMAT_LENGTH);
     }
   }
   else
   {
     memcpy(md.Format,           /* character string format            */
          MQFMT_STRING, (size_t)MQ_FORMAT_LENGTH);
   }

   if (argc > 7)
     md.Encoding = atoi( argv[7] );

   pmo.Options = MQPMO_NO_SYNCPOINT
               | MQPMO_FAIL_IF_QUIESCING;

   /* reset MsgId and CorrelId to get a new one                 */
   memcpy(md.MsgId,    MQMI_NONE, sizeof(md.MsgId) );
   memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId) );

   md.CodedCharSetId = atoi( argv[2] );

   MQPUT(Hcon,                /* connection handle               */
         Hobj,                /* object handle                   */
         &md,                 /* message descriptor              */
         &pmo,                /* default options (datagram)      */
         BufferLen,           /* message length                  */
         buffer,              /* message buffer                  */
         &CompCode,           /* completion code                 */
         &Reason);            /* reason code                     */

   /* report reason, if any */
   if (Reason != MQRC_NONE)
   {
     printf("MQPUT ended with reason code %d\n", Reason);
     printf("Ending Program!\n");
     goto MOD_EXIT;;
   }

   HoldCodedCharSetId = atoi( argv[3] );
   md.CodedCharSetId = HoldCodedCharSetId;
   gmo.Options      = MQGMO_FAIL_IF_QUIESCING + MQGMO_CONVERT+ MQGMO_SYNCPOINT;

   pGetBuffer = (MQBYTE *)malloc( BufferLen );
   if (pGetBuffer == NULL)
   {
     printf("Unable to malloc dynamic memory. Ending program.\n");
     goto MOD_EXIT;;
   }

   MQGET(Hcon,             /* connection handle                */
         Hobj,            /* object handle                  */
         &md,               /* message descriptor               */
         &gmo,              /* get message options              */
         BufferLen,         /* Buffer length                    */
         (MQBYTE *)pGetBuffer, /* message buffer                 */
         &GetMsgLen,       /* message length                   */
         &CompCode,         /* completion code                  */
         &Reason);       /* reason code                      */

   while(Reason == MQRC_TRUNCATED_MSG_FAILED || Reason == MQRC_CONVERTED_STRING_TOO_BIG || Reason == MQRC_CONVERTED_MSG_TOO_BIG)
   {
     if (Reason == MQRC_CONVERTED_STRING_TOO_BIG || Reason == MQRC_CONVERTED_MSG_TOO_BIG)
     {
       GetMsgLen=GetMsgLen*2;

       MQBACK(Hcon,
              &CompCode,
              &Reason);

       if (Reason != MQRC_NONE)
         printf("MQBACK ended with reason code %d\n", Reason);
     }

     free( pGetBuffer );
     BufferLen = GetMsgLen;
     pGetBuffer  = (MQBYTE *)malloc( BufferLen );
     if (pGetBuffer == NULL)
     {
       printf("Unable to malloc dynamic memory. Ending program.\n");
       goto MOD_EXIT;;
     }

     md.Encoding         = MQENC_NATIVE;
     md.CodedCharSetId   = HoldCodedCharSetId; 

     MQGET(Hcon,             /* connection handle                */
           Hobj,            /* object handle                  */
           &md,               /* message descriptor               */
           &gmo,              /* get message options              */
           BufferLen,         /* Buffer length                    */
           (MQBYTE *)pGetBuffer, /* message buffer                 */
           &GetMsgLen,       /* message length                   */
           &CompCode,         /* completion code                  */
           &Reason);       /* reason code                      */

   }

   /* report reason, if any */
   if (Reason != MQRC_NONE)
   {
     printf("MQGET ended with reason code %d\n", Reason);
     printf("Ending Program!\n");
     goto MOD_EXIT;;
   }


   MQCMIT(Hcon,
          &CompCode,
          &Reason);

   /* report reason, if any */
   if (Reason != MQRC_NONE)
     printf("MQCMIT ended with reason code %d\n", Reason);

   printf("The following bytes were PUT with CCSID %d\n", atoi(argv[2]));

   printf("<");

   for (i=0;i<BufferLen;i++)
     printf("%02X",*(buffer+i) & 0xff);

   printf(">\n");

   printf("The following bytes were returned with converted GET with CCSID %d:\n", md.CodedCharSetId);

   printf("<");

   for (i=0;i<GetMsgLen;i++)
     printf("%02X",*(pGetBuffer+i) & 0xff); 

   printf(">\n");

   printf("This is the returned byte string printed as chars (a non isprint char will be shown as '.').\n");

   printf("<");

   for (i=0;i<GetMsgLen;i++)
   {  
     if (isprint(*(pGetBuffer+i))) 
       printf("%c",*(pGetBuffer+i) );
     else
       printf(".");
   }

   printf(">\n");

   C_options = MQCO_NONE;

   MQCLOSE(Hcon,                   /* connection handle            */
           &Hobj,                  /* object handle                */
           C_options,
           &CompCode,              /* completion code              */
           &Reason);               /* reason code                  */

   /* report reason, if any     */
   if (Reason != MQRC_NONE)
   {
     printf("MQCLOSE ended with reason code %d\n", Reason);
     printf("Ending Program!\n");
     goto MOD_EXIT;;
   }

MOD_EXIT:

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM if not already connected                 */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %d\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /* END OF mqcpcnvt                                                */
   /*                                                                */
   /******************************************************************/
   printf("mqcpcnvt end\n");
   return(0);
 }

long ConvertHexToDecLen2(char HexStr[])
{
   int j, len;
   long base16, DecVl;

   base16=1;

   DecVl=0;

   len=1;

   *HexStr=(char) toupper(*HexStr);
   *(HexStr+1)=(char) toupper(*(HexStr+1));
   *(HexStr+2)=(char) toupper(*(HexStr+2));
   *(HexStr+3)=(char) toupper(*(HexStr+3));
   *(HexStr+4)=(char) toupper(*(HexStr+4));
   *(HexStr+5)=(char) toupper(*(HexStr+5));
   *(HexStr+6)=(char) toupper(*(HexStr+6));
   *(HexStr+7)=(char) toupper(*(HexStr+7));

   for (; len > -1; len--)
      {
       switch (HexStr[len])  {
       case '0':
          j = 0;
          break;
       case '1':
          j = 1;
          break;
       case '2':
          j = 2;
          break;
       case '3':
          j = 3;
          break;
       case '4':
          j = 4;
          break;
       case '5':
          j = 5;
          break;
       case '6':
          j = 6;
          break;
       case '7':
          j = 7;
          break;
       case '8':
          j = 8;
          break;
       case '9':
          j = 9;
          break;
       case 'A':
          j = 10;
          break;
       case 'B':
          j = 11;
          break;
       case 'C':
          j = 12;
          break;
       case 'D':
          j = 13;
          break;
       case 'E':
          j = 14;
          break;
       case 'F':
          j = 15;
          break;
       }

       DecVl = (j * base16) + DecVl;
       base16 = base16 * 16;
      }

   return DecVl;
}
